import os

###--------------------------------------------------------------------------------------------------###
###--------------------------------------------------------------------------------------------------###

#GHOST Processing Configuration File 
ghost_version = '1.2'

#define the data root directory (i.e. where the observations are stored)
data_root = '/gpfs/projects/bsc32/AC_cache/obs/ghost/' 

#define the processing root directory (i.e. where the processing is being run from)
process_root = os.getcwd().split('/run')[0]

#define the QOS (Quality of Service) used to manage jobs on the SLURM system
#it is possible to define the qos of the pipeline/stage levels of submission
#OPTIONS: bsc_es (max walltime of 2 days), prace (max walltime of 3 days), debug (max walltime of 2 hours)
qos_pipeline = 'debug'
qos_stage = 'debug'

#define the chunk size to process GHOST stage tasks (i.e. the size of CPU chunks used to process tasks)
#this should not exceed the number of CPUs per node on machine (160 virtual cores/40 physical cores on power9, 48 physical cores on marenostrum4)
chunk_size = 20

#define job array limit to process GHOST stage tasks (i.e. the number of CPU chunks allowed to be submitted simultaneously)
job_array_limit = 100

#define if multithreading should be used when processing tasks (i.e. use all threads per physical CPU)
multithreading = False

###--------------------------------------------------------------------------------------------------###
###--------------------------------------------------------------------------------------------------###
# NETWORKS (list of networks whose data you wish to process)
# VALID OPTIONS: CANADA_NAPS, CAPMoN, EANET, EBAS, EEA_AIRBASE, EEA_AQ_eReporting, NCDC_ISD, SEARCH,
#               US_EPA_AQS, US_EPA_CASTNET, WMO_GAW

networks = ['NCDC_ISD']

# PARAMETERS (list of parameters that you wish to process)
# Gases:                     O3, NO, NO2, CO, SO2, NH3, HNO3, PAN, ISOPRENE
# Aerosol - bulk mass:       PM10, PM2.5, PM1
# Aerosol - bulk speciated:  Al, As, BC, C, C_corrected, Ca++, Cd, Cl-, Co, Cr, Cu, EC, Fe, Hg, K+, Mg++, Mn, Na+, NH4+,
#                           Ni, NO3-, OC, OC_corrected, Pb, Se, SO4--, SO4--_NSS, V, Zn
# Aerosol - PM10 speciated:  PM10_Al, PM10_As, PM10_BC, PM10_C, PM10_C_corrected, PM10_Ca++, PM10_Cd, PM10_Cl-, PM10_Co,
#                           PM10_Cr, PM10_Cu, PM10_EC, PM10_Fe, PM10_Hg, PM10_K+, PM10_Mg++, PM10_Mn, PM10_Na+,
#                           PM10_NH4+, PM10_Ni, PM10_NO3-, PM10_OC, PM10_OC_corrected, PM10_Pb, PM10_Se, PM10_SO4--,
#                           PM10_SO4--_NSS, PM10_V, PM10_Zn
# Aerosol - PM2.5 speciated: PM2.5_Al, PM2.5_As, PM2.5_BC, PM2.5_C, PM2.5_C_corrected, PM2.5_Ca++, PM2.5_Cd, PM2.5_Cl-,
#                           PM2.5_Co, PM2.5_Cr, PM2.5_Cu, PM2.5_EC, PM2.5_Fe, PM2.5_Hg, PM2.5_K+, PM2.5_Mg++, PM2.5_Mn,
#                           PM2.5_Na+, PM2.5_NH3, PM2.5_NH4+, PM2.5_Ni, PM2.5_NO3-, PM2.5_OC, PM2.5_OC_corrected,
#                           PM2.5_Pb, PM2.5_Se, PM2.5_SO4--, PM2.5_SO4--_NSS, PM2.5_V, PM2.5_Zn
# Aerosol - PM1 speciated:   PM1_Al, PM1_As, PM1_BC, PM1_C, PM1_C_corrected, PM1_Ca++, PM1_Cd, PM1_Cl-, PM1_Co, PM1_Cr,
#                           PM1_Cu, PM1_EC, PM1_Fe, PM1_Hg, PM1_K+, PM1_Mg++, PM1_Mn, PM1_Na+, PM1_NH4+, PM1_Ni,
#                           PM1_NO3-, PM1_OC, PM1_OC_corrected, PM1_Pb, PM1_Se, PM1_SO4--, PM1_SO4--_NSS, PM1_V, PM1_Zn
# Meteorology:               WND_DIR, WND_DIR_10M, WND_SPD, WND_SPD_10M, CEILING_HEIGHT, VIS_DIST, TEMP, TEMP_2M, DEW_PT, 
#                           DEW_PT_2M, SLP, PREC_ACCUM, SNOW_ACCUM, SNOW_DEPTH, PRES, PRES_2M, PRES_10M, SST, SOIL_TEMP, 
#                           SOIL_TEMP_10CM, SOIL_TEMP_40CM, SOIL_TEMP_100CM, SOIL_TEMP_200CM, CLOUD_CVG, CLOUD_CVG_FRAC, 
#                           AVG_RH

#parameters = ['O3', 'NO', 'NO2', 'CO', 'SO2', 'NH3', 'HNO3', 'PAN', 'ISOPRENE',
#              'PM10', 'PM2.5', 'PM1',
#              'Al', 'As', 'BC', 'C', 'C_corrected', 'Ca++', 'Cd', 'Cl-', 'Co', 'Cr', 'Cu', 'EC', 'Fe', 'Hg', 'K+', 'Mg++', 'Mn', 'Na+', 'NH4+', 'Ni', 'NO3-', 'OC', 'OC_corrected', 'Pb', 'Se', 'SO4--', 'SO4--_NSS', 'V', 'Zn',
#              'PM10_Al', 'PM10_As', 'PM10_BC', 'PM10_C', 'PM10_C_corrected', 'PM10_Ca++', 'PM10_Cd', 'PM10_Cl-', 'PM10_Co', 'PM10_Cr', 'PM10_Cu', 'PM10_EC', 'PM10_Fe', 'PM10_Hg', 'PM10_K+', 'PM10_Mg++', 'PM10_Mn', 'PM10_Na+', 'PM10_NH4+', 'PM10_Ni', 'PM10_NO3-', 'PM10_OC', 'PM10_OC_corrected', 'PM10_Pb', 'PM10_Se', 'PM10_SO4--', 'PM10_SO4--_NSS', 'PM10_V', 'PM10_Zn',
#              'PM2.5_Al', 'PM2.5_As', 'PM2.5_BC', 'PM2.5_C', 'PM2.5_C_corrected', 'PM2.5_Ca++', 'PM2.5_Cd', 'PM2.5_Cl-', 'PM2.5_Co', 'PM2.5_Cr', 'PM2.5_Cu', 'PM2.5_EC', 'PM2.5_Fe', 'PM2.5_Hg', 'PM2.5_K+', 'PM2.5_Mg++', 'PM2.5_Mn', 'PM2.5_Na+', 'PM2.5_NH3', 'PM2.5_NH4+', 'PM2.5_Ni', 'PM2.5_NO3-', 'PM2.5_OC', 'PM2.5_OC_corrected', 'PM2.5_Pb', 'PM2.5_Se', 'PM2.5_SO4--', 'PM2.5_SO4--_NSS', 'PM2.5_V', 'PM2.5_Zn',
#              'PM1_Al', 'PM1_As', 'PM1_BC', 'PM1_C', 'PM1_C_corrected', 'PM1_Ca++', 'PM1_Cd', 'PM1_Cl-', 'PM1_Co', 'PM1_Cr', 'PM1_Cu', 'PM1_EC', 'PM1_Fe', 'PM1_Hg', 'PM1_K+', 'PM1_Mg++', 'PM1_Mn', 'PM1_Na+', 'PM1_NH4+', 'PM1_Ni', 'PM1_NO3-', 'PM1_OC', 'PM1_OC_corrected', 'PM1_Pb', 'PM1_Se', 'PM1_SO4--', 'PM1_SO4--_NSS', 'PM1_V', 'PM1_Zn',
#              'WND_DIR', 'WND_DIR_10M', 'WND_SPD', 'WND_SPD_10M', 'CEILING_HEIGHT', 'VIS_DIST', 'TEMP', 'TEMP_2M', 'DEW_PT', 'DEW_PT_2M', 'SLP', 'PREC_ACCUM', 'SNOW_ACCUM', 'SNOW_DEPTH', 'PRES', 'PRES_2M', 'PRES_10M', 'SST', 'SOIL_TEMP', 'SOIL_TEMP_10CM', 'SOIL_TEMP_40CM', 'SOIL_TEMP_100CM', 'SOIL_TEMP_200CM', 'CLOUD_CVG', 'CLOUD_CVG_FRAC', 'AVG_RH']
parameters = ['TEMP']

#START_DATE = YYYYMM (start year of processed data file - should be fixed as '197001')
start_date = '197001'

#END_DATE   = YYYYMM (end year of processed data file - should be first month of current year)
end_date = '201901'

#TEMPORAL_RESOLUTIONS = H, D, M, HI (temporal resolution desired of merged output netCDF files: hourly, daily, monthly, hourly instantaneous)       
temporal_resolutions = ['HI']

#STAGES_TO_PROCESS = (list of GHOST stages that you wish to process)

#PARSING OF NETWORK DATA FILES PARSING TO STATION SPECIFIC .npz FILES
#  S1 = Raw network files parse
#  S2 = Concatenate processed data files 
#  S3 = Split stations by significantly changing locational information
#  S4 = Parse globally gridded metadata
#  S5 = Quality control
#  S6 = Process classifications

#TEMPORALLY GRIDDING CONCATENATION OF STATIONS TO netCDF
#  T1 = Temporally grid by station reference
#  T2 = Group temporally gridded station data in monthly netCDFs

#stages_to_process = ['S1','S2','S3','S4','S5','S6','T1','T2']
stages_to_process = ['S1','S2','S3','S4','S5','S6','T1']
###--------------------------------------------------------------------------------------------------###
###--------------------------------------------------------------------------------------------------###
